@@ -3,7 +3,7 @@ context("sym")
 test_that("ensym() fails with calls", {
   capture_sym <- function(arg) ensym(arg)
   expect_identical(capture_sym(foo), quote(foo))
-  expect_error(capture_sym(foo(bar)), "Must supply a symbol")
+  expect_error(capture_sym(foo(bar)), "Only strings can be converted to symbols")
 })
 
 test_that("ensym() supports strings and symbols", {
@@ -27,6 +27,11 @@ test_that("is_symbol() matches `name`", {
   expect_false(is_symbol(sym("foo"), "bar"))
 })
 
+test_that("is_symbol() matches any name in a vector", {
+  expect_false(is_symbol(quote(C), letters))
+  expect_true(is_symbol(quote(c), letters))
+})
+
 test_that("must supply strings to sym()", {
   expect_error(sym(letters), "strings")
   expect_error(sym(1:2), "strings")
