@@ -6,11 +6,6 @@ test_that("vector is modified", {
   expect_equal(out, list(1, b = 20, c = "30", 4, 5, 6))
 })
 
-test_that("are_na() requires vector input but not is_na()", {
-  expect_error(are_na(base::eval), "must be a vector")
-  expect_false(is_na(base::eval))
-})
-
 test_that("seq2() creates increasing sequences", {
   expect_identical(seq2(2, 3), 2:3)
   expect_identical(seq2(3, 2), int())
@@ -20,3 +15,8 @@ test_that("seq2_along() creates increasing sequences", {
   expect_identical(seq2_along(3, 1:2), int())
   expect_identical(seq2_along(-1, 1:2), -1:2)
 })
+
+test_that("seq2() fails with non-scalar inputs", {
+  expect_error(seq2(int(), 1), "must be length one")
+  expect_error(seq2(1, int()), "must be length one")
+})
